<?php  if (!defined('BASEPATH')) exit('No direct script access allowed');

/**
* MY_Output
* 
* Modified _write_cache and _display_cache to allow for header caching
* Added clear_cache and _delete_cache to allow for deletion of cached resources
* works with CodeIgniter 1.7.0
*
* @package      CodeIgniter
* @subpackage   Libraries
* @category     Output
* @author       Arjen van Bochoven
* @author       Prateek Rungta
* @link         http://codeigniter.com/wiki/Cache_headers
* @link         http://prateekrungta.com/bitsnpieces/codeigniter-libraries
* @version      1.0
*/

class MY_Output extends CI_Output {

	/**
	 * Write a Cache File
	 *
	 * @access  public
	 * @return  void
	 * @author  Arjen van Bochoven
	 */	   
	function _write_cache($output)
	{
		$CI =& get_instance();	  
		$path = $CI->config->item('cache_path');

		$cache_path = ($path == '') ? BASEPATH.'cache/' : $path;
	
		if ( ! is_dir($cache_path) OR ! is_really_writable($cache_path))
		{
			return;
		}
	
		$uri =	$CI->config->item('base_url').
				$CI->config->item('index_page').
				$CI->uri->uri_string();
	
		$cache_path .= md5($uri);

		if ( ! $fp = @fopen($cache_path, FOPEN_WRITE_CREATE_DESTRUCTIVE))
		{
			log_message('error', "Unable to write cache file: ".$cache_path);
			return;
		}
	
		$expire = time() + ($this->cache_expiration * 60);
			
		if (flock($fp, LOCK_EX))
		{
			fwrite($fp, serialize(array('exp' => $expire, 'headers' => $this->headers, 'output' => $output)));
			flock($fp, LOCK_UN);
		}
		else
		{
			log_message('error', "Unable to secure a file lock for file at: ".$cache_path);
			return;
		}
		fclose($fp);
		@chmod($cache_path, DIR_WRITE_MODE);

		log_message('debug', "Cache file written: ".$cache_path);
	}

	// --------------------------------------------------------------------

	/**
	 * Update/serve a cached file
	 *
	 * @access  public
	 * @return  void
	 * @author  Arjen van Bochoven
	 */	   
	function _display_cache(&$CFG, &$URI)
	{
		$cache_path = ($CFG->item('cache_path') == '') ? BASEPATH.'cache/' : $CFG->item('cache_path');
		
		if ( ! is_dir($cache_path) OR ! is_really_writable($cache_path))
		{
			return FALSE;
		}
	
		// Build the file path.	 The file name is an MD5 hash of the full URI
		$uri =	$CFG->item('base_url').
				$CFG->item('index_page').
				$URI->uri_string;
			
		$filepath = $cache_path.md5($uri);
	
		if ( ! @file_exists($filepath))
		{
			return FALSE;
		}

		if ( ! $fp = @fopen($filepath, FOPEN_READ))
		{
			return FALSE;
		}
		
		flock($fp, LOCK_SH);
		
		$cache = '';
		if (filesize($filepath) > 0)
		{
			$cache = fread($fp, filesize($filepath));
		}
	
		flock($fp, LOCK_UN);
		fclose($fp);
		
		// Restore the cache array
		$cache = unserialize($cache);
		
		// Validate cache file
		if ( ! isset($cache['exp'] ) OR ! isset($cache['headers']) OR ! isset($cache['output']))
		{
			return FALSE;
		}
		
		// Has the file expired? If so we'll delete it.
		if (time() >= trim($cache['exp']))
		{		  
			@unlink($filepath);
			log_message('debug', "Cache file has expired. File deleted");
			return FALSE;
		}
		
		// Restore headers
		$this->headers = $cache['headers'];
				
		// Display the cache
		$this->_display($cache['output']);
		log_message('debug', "Cache file is current. Sending it to browser.");		  
		return TRUE;
		
	}

	// --------------------------------------------------------------------
	
	/**
	 * Request removal of a cached page
	 *
	 * @access  public
	 * @return  void
	 * @author  Prateek Rungta
	 */
	function clear_cache($uri)
	{
		// The CI uri_string begins with a slash
		if ($uri[0] != '/') $uri = '/'.$uri;
		
		// Delete cache for the uri
		$this->_delete_cache($uri);
		
		// Delete the other version (trailing/non-traling)
		if ($uri[strlen($uri)-1] == '/')
			$this->_delete_cache(substr($uri, 0, -1));
		else
			$this->_delete_cache($uri.'/');
	}
	
	// --------------------------------------------------------------------
	
	/**
	 * Delete a cached file
	 *
	 * @access  private
	 * @return  void
	 * @author  Prateek Rungta
	 */	   
	function _delete_cache($uri_string)
	{
		$CI =& get_instance();	  
		$path = $CI->config->item('cache_path');

		$cache_path = ($path == '') ? BASEPATH.'cache/' : $path;
	
		if ( ! is_dir($cache_path) OR ! is_really_writable($cache_path))
		{
			return;
		}
		
		// Build hash string
		$uri =	$CI->config->item('base_url').
				$CI->config->item('index_page').
				$uri_string;
		
		// Filename
		$filepath = $cache_path.md5($uri);
		
		if ( ! @file_exists($filepath))
		{
			return FALSE;
		}
		
		// Delete the cached file
		@unlink($filepath);
		log_message('debug', "Cache file deleted on request.");
		return TRUE;	
	}

	// --------------------------------------------------------------------
	
}

/* End of file */